"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildExplicitDependencies = void 0;
const project_graph_1 = require("../../../../config/project-graph");
const explicit_project_dependencies_1 = require("./explicit-project-dependencies");
const explicit_package_json_dependencies_1 = require("./explicit-package-json-dependencies");
function buildExplicitDependencies(jsPluginConfig, ctx, builder) {
    if (totalNumberOfFilesToProcess(ctx) === 0)
        return;
    let dependencies = [];
    if (jsPluginConfig.analyzeSourceFiles === undefined ||
        jsPluginConfig.analyzeSourceFiles === true) {
        let tsExists = false;
        try {
            require.resolve('typescript');
            tsExists = true;
        }
        catch (_a) { }
        if (tsExists) {
            dependencies = dependencies.concat((0, explicit_project_dependencies_1.buildExplicitTypeScriptDependencies)(builder.graph, ctx.filesToProcess));
        }
    }
    if (jsPluginConfig.analyzePackageJson === undefined ||
        jsPluginConfig.analyzePackageJson === true) {
        dependencies = dependencies.concat((0, explicit_package_json_dependencies_1.buildExplicitPackageJsonDependencies)(ctx.nxJsonConfiguration, ctx.projectsConfigurations, builder.graph, ctx.filesToProcess));
    }
    dependencies.forEach((r) => addDependency(builder, r));
}
exports.buildExplicitDependencies = buildExplicitDependencies;
function totalNumberOfFilesToProcess(ctx) {
    let totalNumOfFilesToProcess = 0;
    Object.values(ctx.filesToProcess).forEach((t) => (totalNumOfFilesToProcess += t.length));
    return totalNumOfFilesToProcess;
}
function addDependency(builder, dependency) {
    if (dependency.type === project_graph_1.DependencyType.static) {
        builder.addStaticDependency(dependency.sourceProjectName, dependency.targetProjectName, dependency.sourceProjectFile);
    }
    else {
        builder.addDynamicDependency(dependency.sourceProjectName, dependency.targetProjectName, dependency.sourceProjectFile);
    }
}
